@@ -62,6 +62,9 @@ impl Default for PatternDirection {
 #[cfg_attr(feature = "serialize", serde(rename_all = "camelCase"))]
 #[derive(Clone, Debug)]
 pub struct CutPiece {
+    /// Quantity of this cut piece.
+    pub quantity: usize,
+
     /// ID to be used by the caller to match up result cut pieces
     /// with the original cut piece. This ID has no meaning to the
     /// optimizer so it can be set to `None` if not needed.
@@ -119,18 +122,6 @@ impl PartialEq for UsedCutPiece {
 }
 impl Eq for UsedCutPiece {}
 
-impl From<CutPieceWithId> for CutPiece {
-    fn from(cut_piece: CutPieceWithId) -> Self {
-        Self {
-            external_id: cut_piece.external_id,
-            width: cut_piece.width,
-            length: cut_piece.length,
-            can_rotate: cut_piece.can_rotate,
-            pattern_direction: cut_piece.pattern_direction,
-        }
-    }
-}
-
 impl From<UsedCutPiece> for CutPieceWithId {
     fn from(used_cut_piece: UsedCutPiece) -> Self {
         let (width, length, pattern_direction) = if used_cut_piece.is_rotated {
@@ -263,6 +254,9 @@ pub struct ResultStockPiece {
 
     /// Waste pieces that remain after cutting the cut pieces.
     pub waste_pieces: Vec<Rect>,
+
+    /// Price of stock piece.
+    pub price: usize,
 }
 
 /// A rectangle
@@ -443,6 +437,21 @@ where
     blade_width: usize,
 }
 
+impl<'a, B> Clone for OptimizerUnit<'a, B>
+where
+    B: Bin + Clone,
+{
+    fn clone(&self) -> Self {
+        Self {
+            bins: self.bins.clone(),
+            possible_stock_pieces: self.possible_stock_pieces,
+            available_stock_pieces: self.available_stock_pieces.to_vec(),
+            unused_cut_pieces: self.unused_cut_pieces.clone(),
+            blade_width: self.blade_width,
+        }
+    }
+}
+
 impl<'a, B> OptimizerUnit<'a, B>
 where
     B: Bin,
@@ -640,23 +649,15 @@ where
         R: Rng + ?Sized,
         B: Clone,
     {
-        let cross_dest = if self.bins.is_empty() {
-            0
-        } else {
-            rng.gen_range(0..self.bins.len() + 1)
-        };
+        // If there aren't multiple bins we can't do a crossover, so just return a clone of this
+        // unit.
+        if self.bins.len() < 2 && other.bins.len() < 2 {
+            return self.clone();
+        }
 
-        let cross_src_start = if other.bins.is_empty() {
-            0
-        } else {
-            rng.gen_range(0..other.bins.len())
-        };
-
-        let cross_src_end = if other.bins.is_empty() {
-            0
-        } else {
-            rng.gen_range(cross_src_start + 1..other.bins.len() + 1)
-        };
+        let cross_dest = rng.gen_range(0..=self.bins.len());
+        let cross_src_start = rng.gen_range(0..other.bins.len());
+        let cross_src_end = rng.gen_range(cross_src_start + 1..=other.bins.len());
 
         let mut new_unit = OptimizerUnit {
             // Inject bins between crossing sites of other.
@@ -800,6 +801,7 @@ pub enum Error {
 }
 fn no_fit_for_cut_piece_error(cut_piece: &CutPieceWithId) -> Error {
     Error::NoFitForCutPiece(CutPiece {
+        quantity: 1,
         external_id: cut_piece.external_id,
         width: cut_piece.width,
         length: cut_piece.length,
@@ -901,16 +903,19 @@ impl Optimizer {
 
     /// Add a desired cut piece that you need cut from a stock piece.
     pub fn add_cut_piece(&mut self, cut_piece: CutPiece) -> &mut Self {
-        let cut_piece = CutPieceWithId {
-            id: self.cut_pieces.len(),
-            external_id: cut_piece.external_id,
-            width: cut_piece.width,
-            length: cut_piece.length,
-            pattern_direction: cut_piece.pattern_direction,
-            can_rotate: cut_piece.can_rotate,
-        };
+        for _ in 0..cut_piece.quantity {
+            let cut_piece = CutPieceWithId {
+                id: self.cut_pieces.len(),
+                external_id: cut_piece.external_id,
+                width: cut_piece.width,
+                length: cut_piece.length,
+                pattern_direction: cut_piece.pattern_direction,
+                can_rotate: cut_piece.can_rotate,
+            };
+
+            self.cut_pieces.push(cut_piece);
+        }
 
-        self.cut_pieces.push(cut_piece);
         self
     }
 
@@ -1002,7 +1007,7 @@ impl Optimizer {
             // here. Each stock size will be optmized separately below.
             // Note: it's safe to assume `self.cut_pieces` isn't empty because
             // that's checked at the beginning of this function.
-            Err(Error::NoFitForCutPiece(self.cut_pieces[0].clone().into()))
+            Err(no_fit_for_cut_piece_error(&self.cut_pieces[0]))
         };
 
         // Optimize each stock size separately and see if any have better result than
